# -*- coding: utf-8 -*-

"""
   Pinch.Controllers.TicketController

   This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.APIHelper import APIHelper
from Pinch.APIException import APIException
from Pinch.Configuration import Configuration
from Pinch.Http.HttpRequest import HttpRequest
from Pinch.Http.HttpResponse import HttpResponse
from Pinch.Http.RequestsClient import RequestsClient
from Pinch.Controllers.BaseController import BaseController
from Pinch.CustomAuthUtility import CustomAuthUtility

from Pinch.Models.Ticket import Ticket


class TicketController(BaseController):

    """A Controller to access Endpoints in the Pinch API."""

    def __init__(self, http_client = None):
        """Constructor which allows a different HTTP client for this controller."""
        BaseController.__init__(self, http_client)

    def list(self):
        """Does a GET request to /tickets.

        List all the opened tickets of every clients you are working for

        Returns:
            list of Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets"
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.get(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass
        
        # Try to cast response to list of desired type
        if isinstance(_response.raw_body, list):
            # Response is already in a list, return the list of objects 
            value = list()
            for item in _response.raw_body:
                try:
                    value.append(Ticket.from_dictionary(item))
                except Exception:
                    raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)
                    
            return value

    def accept_intervention(self,
                            ticket_id):
        """Does a POST request to /tickets/{ticket_id}/accept.

        This method returns no result but the status code tells you if the
        operation succedded

        Args:
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/accept"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def set_intervention_date(self,
                              intervention_date,
                              ticket_id):
        """Does a POST request to /tickets/{ticket_id}/set_intervention_date.

        TODO: type endpoint description here.

        Args:
            intervention_date (DateTime): TODO: type description here.
                Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if intervention_date == None:
            raise ValueError("Required parameter 'intervention_date' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/set_intervention_date"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare form parameters
        _form_parameters = {
            "intervention_date": intervention_date
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, parameters=_form_parameters)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def upload_quote(self,
                     file,
                     ticket_id):
        """Does a POST request to /tickets/{ticket_id}/quote_upload.

        TODO: type endpoint description here.

        Args:
            file (string): TODO: type description here. Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if file == None:
            raise ValueError("Required parameter 'file' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/quote_upload"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })

        _files = {
            "file": file

        }
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, files=_files)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def send_message(self,
                     body,
                     ticket_id):
        """Does a POST request to /tickets/{ticket_id}/message.

        TODO: type endpoint description here.

        Args:
            body (string): TODO: type description here. Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if body == None:
            raise ValueError("Required parameter 'body' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/message"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare form parameters
        _form_parameters = {
            "body": body
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, parameters=_form_parameters)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def declare_intervention_done(self,
                                  ticket_id,
                                  intervention_date = None):
        """Does a POST request to /tickets/{ticket_id}/intervention_done.

        TODO: type endpoint description here.

        Args:
            ticket_id (string): TODO: type description here. Example: 
            intervention_date (DateTime, optional): TODO: type description
                here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/intervention_done"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare form parameters
        _form_parameters = {
            "intervention_date": intervention_date
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, parameters=_form_parameters)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def upload_document(self,
                        file,
                        ticket_id):
        """Does a POST request to /tickets/{ticket_id}/document_upload.

        The document should not be an invoice nor a quote

        Args:
            file (string): TODO: type description here. Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if file == None:
            raise ValueError("Required parameter 'file' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/document_upload"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })

        _files = {
            "file": file

        }
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, files=_files)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def upload_picture(self,
                       file,
                       ticket_id):
        """Does a POST request to /tickets/{ticket_id}/picture_upload.

        TODO: type endpoint description here.

        Args:
            file (string): TODO: type description here. Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if file == None:
            raise ValueError("Required parameter 'file' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/picture_upload"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })

        _files = {
            "file": file

        }
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, files=_files)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def get(self,
                ticket_id):
        """Does a GET request to /tickets/{ticket_id}.

        TODO: type endpoint description here.

        Args:
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            Ticket: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.get(_query_url, headers=_headers)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Try to convert response to JSON
        try:
            _response.raw_body = APIHelper.json_deserialize(_response.raw_body)
        except:
            pass

        # Try to cast response to desired type
        if isinstance(_response.raw_body, dict):
            # Response is already in a dictionary, return the object 
            try:
                return Ticket.from_dictionary(_response.raw_body)
            except Exception:
                raise APIException("Invalid JSON returned.", _response.status_code, _response.raw_body)

    def upload_invoice(self,
                       file,
                       ticket_id):
        """Does a POST request to /tickets/{ticket_id}/invoice_upload.

        TODO: type endpoint description here.

        Args:
            file (string): TODO: type description here. Example: 
            ticket_id (string): TODO: type description here. Example: 

        Returns:
            string: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Validate required parameters
        if file == None:
            raise ValueError("Required parameter 'file' cannot be None.")
        elif ticket_id == None:
            raise ValueError("Required parameter 'ticket_id' cannot be None.")

        # The base uri for api requests
        _query_builder = Configuration.BASE_URI
 
        # Prepare query string for API call
        _query_builder += "/tickets/{ticket_id}/invoice_upload"

        # Process optional template parameters
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            "ticket_id": ticket_id
        })

        _files = {
            "file": file

        }
        
        # Validate and preprocess url
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            "user-agent": "APIMATIC 2.0",
            "X-API-TOKEN": Configuration.x_api_token,
            "X-API-EMAIL": Configuration.x_api_email
        }

        # Prepare the API call.
        _http_request = self.http_client.post(_query_url, headers=_headers, files=_files)

        #append custom auth authorization
        CustomAuthUtility.appendCustomAuthParams(_http_request)

        # Invoke the API call  to fetch the response.
        _response = self.http_client.execute_as_string(_http_request)

        # Global error handling using HTTP status codes.
        self.validate_response(_response)    

        # Return appropriate type
        return _response.raw_body





from Pinch.Controllers.BaseController import *
from Pinch.Controllers.WebhookTypeController import *
from Pinch.Controllers.WebhookController import *
from Pinch.Controllers.TicketController import *



"""
    Pinch

    This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.APIException import APIException
from Pinch.Http.HttpRequest import HttpRequest
from Pinch.Http.HttpResponse import HttpResponse
from Pinch.Http.HttpMethodEnum import HttpMethodEnum

class HttpClient(object):

    """An interface for the methods that an HTTP Client must implement
    
    This class contains methods that will all return an exception. This
	class should not be instantiated but should be used as a base class
	for HTTP Client classes.
    
    """
    
    def execute_as_string(self, request):
        """Execute a given HttpRequest to get a string response back
       
        Args:
            request (HttpRequest): The given HttpRequest to execute.
            
        Returns:
            HttpResponse: The response of the HttpRequest.
            
        """
        raise NotImplementedError("Please Implement this method")
    
    def execute_as_binary(self, request):
        """Execute a given HttpRequest to get a binary response back
       
        Args:
            request (HttpRequest): The given HttpRequest to execute.
            
        Returns:
            HttpResponse: The response of the HttpRequest.
            
        """
        raise NotImplementedError("Please Implement this method")
    
    def convert_response(self, request):
        """Converts the Response object of the HttpClient into an
		HttpResponse object.
       
        Args:
            response (dynamic): The original response object.
            
        Returns:
            HttpResponse: The converted HttpResponse object.
            
        """
        raise NotImplementedError("Please Implement this method")

    def get(self, query_url,
            headers = None,
            query_parameters = None,
            username = None,
            password = None):
        """Create a simple GET HttpRequest object for the given parameters
        
        Args:
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
            
        Returns:
            HttpRequest: The generated HttpRequest for the given paremeters.
            
        """
        return HttpRequest(HttpMethodEnum.GET,
                           query_url,
                           headers,
                           query_parameters,
                           None,
						   None,
                           username,
                           password)
    
    def post(self, query_url,
            headers = None,
            query_parameters = None,
            parameters = None,
            files = None,
            username = None,
            password = None):
        """Create a simple POST HttpRequest object for the given parameters
        
        Args:
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            parameters (dict, optional): Form or body parameters to be included in the body.
			files (dict, optional): Files to be sent with the request.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
            
        Returns:
            HttpRequest: The generated HttpRequest for the given paremeters.
            
        """
        return HttpRequest(HttpMethodEnum.POST,
                           query_url,
                           headers,
                           query_parameters,
                           parameters,
                           files,
                           username,
                           password)
    
    def put(self, query_url,
            headers = None,
            query_parameters = None,
            parameters = None,
            files = None,
            username = None,
            password = None):
        """Create a simple PUT HttpRequest object for the given parameters
        
        Args:
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            parameters (dict, optional): Form or body parameters to be included in the body.
			files (dict, optional): Files to be sent with the request.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
            
        Returns:
            HttpRequest: The generated HttpRequest for the given paremeters.
            
        """
        return HttpRequest(HttpMethodEnum.PUT,
                           query_url,
                           headers,
                           query_parameters,
                           parameters,
                           files,
                           username,
                           password)
    
    def patch(self, query_url,
            headers = None,
            query_parameters = None,
            parameters = None,
            files = None,
            username = None,
            password = None):
        """Create a simple PATCH HttpRequest object for the given parameters
        
        Args:
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            parameters (dict, optional): Form or body parameters to be included in the body.
			files (dict, optional): Files to be sent with the request.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
            
        Returns:
            HttpRequest: The generated HttpRequest for the given paremeters.
            
        """
        return HttpRequest(HttpMethodEnum.PATCH,
                           query_url,
                           headers,
                           query_parameters,
                           parameters,
                           files,
                           username,
                           password)
    
    def delete(self, query_url,
            headers = None,
            query_parameters = None,
            parameters = None,
            files = None,
            username = None,
            password = None):
        """Create a simple DELETE HttpRequest object for the given parameters
        
        Args:
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            parameters (dict, optional): Form or body parameters to be included in the body.
			files (dict, optional): Files to be sent with the request.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
            
        Returns:
            HttpRequest: The generated HttpRequest for the given paremeters.
            
        """
        return HttpRequest(HttpMethodEnum.DELETE,
                           query_url,
                           headers,
                           query_parameters,
                           parameters,
                           files,
                           username,
                           password)



# -*- coding: utf-8 -*-

"""
   Pinch.Models.Person
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel

class Person(BaseModel):

    """Implementation of the 'Person' model.

    TODO: type model description here.

    Attributes:
        firstname (string): TODO: type description here.
        lastname (string): TODO: type description here.
        home_phone_number (string): The landline phone number of the resident
            or manager
        mobile_phone_number (string): TODO: type description here.
        role (string): Caretaker, Resident, Manager, ThirdParty

    """

    def __init__(self, 
                 firstname = None,
                 lastname = None,
                 home_phone_number = None,
                 mobile_phone_number = None,
                 role = None):
        """Constructor for the Person class"""
        
        # Initialize members of the class
        self.firstname = firstname
        self.lastname = lastname
        self.home_phone_number = home_phone_number
        self.mobile_phone_number = mobile_phone_number
        self.role = role

        # Create a mapping from Model property names to API property names
        self.names = {
            "firstname": "firstname",
            "lastname": "lastname",
            "home_phone_number": "home_phone_number",
            "mobile_phone_number": "mobile_phone_number",
            "role": "role",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:	
            # Extract variables from the dictionary
            firstname = dictionary.get("firstname")
            lastname = dictionary.get("lastname")
            home_phone_number = dictionary.get("home_phone_number")
            mobile_phone_number = dictionary.get("mobile_phone_number")
            role = dictionary.get("role")
            # Return an object of this model
            return cls(firstname,
                       lastname,
                       home_phone_number,
                       mobile_phone_number,
                       role)



# -*- coding: utf-8 -*-

"""
   Pinch.APIHelper

   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""

import re
import jsonpickle
from requests.utils import quote
from Pinch.Models.BaseModel import BaseModel

class APIHelper(object):

    """A Helper Class for various functions associated with API Calls.
    
    This class contains static methods for operations that need to be
    performed during API requests. All of the methods inside this class are
    static methods, there is no need to ever initialise an instance of this
    class.
    
    """
    
    @staticmethod
    def json_serialize(obj):
        """JSON Serialization of a given object.
       
        Args:
            obj (object): The object to serialise.
            
        Returns:
            str: The JSON serialized string of the object.
            
        """
        if obj is None:
            return None

        # Resolve any Names if it's one of our objects that needs to have this called on
        if isinstance(obj, list):
            value = list()
            for item in obj:
                if isinstance(item, BaseModel):
                    value.append(item.to_dictionary())
                else:
                    value.append(item)
            obj = value
        else:
            if isinstance(obj, BaseModel):
                obj = obj.to_dictionary()

        return jsonpickle.encode(obj, False)

    @staticmethod
    def json_deserialize(json):
        """JSON Deerialization of a given string.
       
        Args:
            json (str): The JSON serialized string to deserialize.
            
        Returns:
            dict: A dictionary representing the data contained in the
                JSON serialized string.
            
        """
        if json is None:
            return None

        return jsonpickle.decode(json)

    @staticmethod
    def append_url_with_template_parameters(url,
                                            parameters):
        """Replaces template parameters in the given url.
       
        Args:
            url (str): The query url string to replace the template parameters.
            parameters (dict): The parameters to replace in the url.
            
        Returns:
            str: Url with replaced parameters.
            
        """
        # Parameter validation
        if url is None:
            raise ValueError("URL is None.")
        if parameters is None:
            return url

        # Iterate and replace parameters
        for key in parameters:
            element = parameters[key]
            replace_value = ""

            # Load parameter value
            if element is None:
                replace_value = ""
            elif isinstance(element, list):
                replace_value = "/".join(str(x) for x in element)
            else:
                replace_value = str(element)

            url = url.replace('{{{0}}}'.format(key),str(replace_value))

        return url

    @staticmethod
    def clean_url(url):
        """Validates and processes the given query Url to clean empty slashes.
       
        Args:
            url (str): The given query Url to process.
            
        Returns:
            str: Clean Url as string.
            
        """
        # Ensure that the urls are absolute
        regex = "^https?://[^/]+"
        match = re.match(regex, url)
        if match is None:
            raise ValueError('Invalid Url format.')

        protocol = match.group(0)
        index = url.find('?')
        query_url = url[len(protocol): index if index != -1 else None]
        query_url = re.sub("//+", "/", query_url)
        parameters = url[index:] if index != -1 else ""

        return protocol + query_url + parameters
        
    @staticmethod
    def form_encode_parameters(form_parameters):       
        """Form encodes a dictionary of form parameters
       
        Args:
            form_parameters (dictionary): The given dictionary which has
            atleast one model to form encode.
            
        Returns:
            dict: A dictionary of form encoded properties of the model.
            
        """
        encoded = dict()
        for key, value in form_parameters.items():
            encoded.update(APIHelper.form_encode(value, key))

        return encoded
    
    @staticmethod
    def form_encode(obj,
                    instanceName):
        """Encodes a model in a form-encoded manner such as person[Name]
       
        Args:
            obj (object): The given Object to form encode.
            instanceName (string): The base name to appear before each entry
                for this object.
            
        Returns:
            dict: A dictionary of form encoded properties of the model.
            
        """
        retval = dict()

        # If we received an object, resolve it's field names.
        if isinstance(obj, BaseModel):
            obj = obj.to_dictionary()  
        
        if obj is None:
            return None           
        elif isinstance(obj, list):
            for index, entry in enumerate(obj):
                retval.update(APIHelper.form_encode(entry, instanceName + "[" + str(index) + "]"))
        elif isinstance(obj, dict):
            for item in obj:
                retval.update(APIHelper.form_encode(obj[item], instanceName + "[" + item + "]"))
        else:
            retval[instanceName] = obj
        
        return retval


# -*- coding: utf-8 -*-

"""
   Pinch.Models.Building
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel

class Building(BaseModel):

    """Implementation of the 'Building' model.

    TODO: type model description here.

    Attributes:
        reference (string): TODO: type description here.
        name (string): TODO: type description here.
        address (string): TODO: type description here.
        zip_code (string): TODO: type description here.
        city (string): TODO: type description here.
        country (string): TODO: type description here.
        latitude (float): TODO: type description here.
        longitude (float): TODO: type description here.

    """

    def __init__(self, 
                 reference = None,
                 name = None,
                 address = None,
                 zip_code = None,
                 city = None,
                 country = None,
                 latitude = None,
                 longitude = None):
        """Constructor for the Building class"""
        
        # Initialize members of the class
        self.reference = reference
        self.name = name
        self.address = address
        self.zip_code = zip_code
        self.city = city
        self.country = country
        self.latitude = latitude
        self.longitude = longitude

        # Create a mapping from Model property names to API property names
        self.names = {
            "reference": "reference",
            "name": "name",
            "address": "address",
            "zip_code": "zip_code",
            "city": "city",
            "country": "country",
            "latitude": "latitude",
            "longitude": "longitude",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:	
            # Extract variables from the dictionary
            reference = dictionary.get("reference")
            name = dictionary.get("name")
            address = dictionary.get("address")
            zip_code = dictionary.get("zip_code")
            city = dictionary.get("city")
            country = dictionary.get("country")
            latitude = dictionary.get("latitude")
            longitude = dictionary.get("longitude")
            # Return an object of this model
            return cls(reference,
                       name,
                       address,
                       zip_code,
                       city,
                       country,
                       latitude,
                       longitude)



"""
    Pinch

    This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
from Pinch.Http.HttpMethodEnum import HttpMethodEnum

class HttpRequest(object):

    """Information about an HTTP Request including its method, headers,
        parameters, URL, and Basic Auth details

    Attributes:
        http_method (HttpMethodEnum): The HTTP Method that this request should
		    perform when called.
        headers (dict): A dictionary of headers (key : value) that should be
		    sent along with the request.
        query_url (string): The URL that the request should be sent to.
        parameters (dict): A dictionary of parameters that are to be sent along
		    with the request in the form body of the request
        username (string): If this request is to use Basic Authentication for
		    authentication then this property represents the corresponding
			username.
        password (string): If this request is to use Basic Authentication for
		    authentication then this property represents the corresponding
			password.

    """

    def __init__(self,
                 http_method,
                 query_url,
                 headers = None,
				 query_parameters = None,
                 parameters = None,
				 files = None,
                 username = None,
                 password = None):
        """Constructor for the HttpRequest class
        
        Args:
            http_method (HttpMethodEnum): The HTTP Method.
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
			query_parameters (dict, optional): Query parameters to add in the URL.
            parameters (dict, optional): Form or body parameters to be included in the body.
			files (dict, optional): Files to be sent with the request.
            username (string, optional): Username for Basic Auth requests.
            password (string, optional): Password for Basic Auth requests.
        
        """
        self.http_method = http_method
        self.query_url = query_url
        self.headers = headers
        self.query_parameters = query_parameters
        self.parameters = parameters
        self.files = files
        self.username = username
        self.password = password



# -*- coding: utf-8 -*-

"""
   Pinch.Models.Ticket
 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""
from Pinch.Models.BaseModel import BaseModel
from Pinch.Models.Person import Person
from Pinch.Models.Building import Building
from Pinch.Models.Unit import Unit
from Pinch.Models.Document import Document

class Ticket(BaseModel):

    """Implementation of the 'Ticket' model.

    TODO: type model description here.

    Attributes:
        id (string): TODO: type description here.
        name (string): TODO: type description here.
        description (string): TODO: type description here.
        contacts (list of Person): TODO: type description here.
        status (string): TODO: type description here.
        building (Building): TODO: type description here.
        unit (Unit): TODO: type description here.
        access (string): TODO: type description here.
        agency (string): TODO: type description here.
        manager (string): TODO: type description here.
        documents (list of Document): TODO: type description here.

    """

    def __init__(self, 
                 id = None,
                 name = None,
                 description = None,
                 contacts = None,
                 status = None,
                 building = None,
                 unit = None,
                 access = None,
                 agency = None,
                 manager = None,
                 documents = None):
        """Constructor for the Ticket class"""
        
        # Initialize members of the class
        self.id = id
        self.name = name
        self.description = description
        self.contacts = contacts
        self.status = status
        self.building = building
        self.unit = unit
        self.access = access
        self.agency = agency
        self.manager = manager
        self.documents = documents

        # Create a mapping from Model property names to API property names
        self.names = {
            "id": "id",
            "name": "name",
            "description": "description",
            "contacts": "contacts",
            "status": "status",
            "building": "building",
            "unit": "unit",
            "access": "access",
            "agency": "agency",
            "manager": "manager",
            "documents": "documents",
        }

    @classmethod
    def from_dictionary(cls, 
                        dictionary):
        """Creates an instance of this model from a dictionary
       
        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.
            
        Returns:
            object: An instance of this structure class.
            
        """

        if dictionary == None:
            return None
        else:   
            # Extract variables from the dictionary
            id = dictionary.get("id")
            name = dictionary.get("name")
            description = dictionary.get("description")
            # Parameter is an array, so we need to iterate through it
            contacts = None
            if dictionary.get("contacts") != None:
                contacts = list()
                for structure in dictionary.get("contacts"):
                    contacts.append(Person.from_dictionary(structure))
            status = dictionary.get("status")
            building = Building.from_dictionary(dictionary.get("building"))
            unit = Unit.from_dictionary(dictionary.get("unit"))
            access = dictionary.get("access")
            agency = dictionary.get("agency")
            manager = dictionary.get("manager")
            # Parameter is an array, so we need to iterate through it
            documents = None
            if dictionary.get("documents") != None:
                documents = list()
                for structure in dictionary.get("documents"):
                    documents.append(Document.from_dictionary(structure))
            # Return an object of this model
            return cls(id,
                       name,
                       description,
                       contacts,
                       status,
                       building,
                       unit,
                       access,
                       agency,
                       manager,
                       documents)



# -*- coding: utf-8 -*-

""" 
   This file was automatically generated for Pinch by APIMATIC v2.0 ( https://apimatic.io ) on 05/13/2016
"""

class BaseModel(object):    

    """All classes of structures inherit this base class.
    
    we using this inheritance relationship to seperate the common
    logic all structure classes use. We also use it to detect if a
    class is a structure class.
    
    """

    def to_dictionary(self):
        """Creates a dictionary representation of a structure class. The 
        keys are taken from the API description and may differ from language
        specific variable names of properties.
            
        Returns:
            dictionary: A dictionary form of the model with properties in 
			their API formats.
            
        """

        dictionary = dict()

        # Loop through all properties in this model
        for name in self.names:
            value = getattr(self, name)                
            if isinstance(value, list):
                # Loop through each item
                dictionary[self.names[name]] = list()
                for item in value:
                    dictionary[self.names[name]].append(item.to_dictionary() if isinstance(item, BaseModel) else item)
            elif isinstance(value, dict):
                # Loop through each item
                dictionary[self.names[name]] = dict()
                for key in value:
                    dictionary[self.names[name]][key] = value[key].to_dictionary() if isinstance(value[key], BaseModel) else value[key]
            else:
                dictionary[self.names[name]] = value.to_dictionary() if isinstance(value, BaseModel) else value

        # Return the result
        return dictionary




from setuptools import setup, find_packages
setup(
  name = 'Pinch',
  packages = find_packages(),
  version = '0.1.5',
  description = 'A simple gem to easily connect to the Inch Housing Intervention Interface',
  author = 'Inch',
  author_email = 'thomas@inchbase.com',
  url = 'https://github.com/Uelb/pinch-python', # use the URL to the github repo
  download_url = 'https://github.com/Uelb/pinch-python/tarball/v0.1.0', # I'll explain this in a second
  keywords = ['company', 'api', 'housing', 'intervention', 'geronimo', 'inch'], # arbitrary keywords
  classifiers = [],
)



"""
    Pinch

    This file was automatically generated for Pinch by APIMATIC BETA v2.0 on 05/13/2016
"""
import requests

from Pinch.Http.HttpClient import HttpClient
from Pinch.Http.HttpResponse import HttpResponse
from Pinch.Http.HttpMethodEnum import HttpMethodEnum

class RequestsClient(HttpClient):

    """An implementation of HttpClient that uses Requests as its HTTP Client
    
    """
    def execute_as_string(self, request):
        """Execute a given HttpRequest to get a string response back
       
        Args:
            request (HttpRequest): The given HttpRequest to execute.
            
        Returns:
            HttpResponse: The response of the HttpRequest.
            
        """	
        response = requests.request(HttpMethodEnum.to_string(request.http_method), 
		                           request.query_url, 
								   headers=request.headers,
		                           params=request.query_parameters, 
                                   data=request.parameters,
								   files=request.files,
								   auth=(request.username, request.password))

        return self.convert_response(response, False)
    
    def execute_as_binary(self, request):
        """Execute a given HttpRequest to get a binary response back
       
        Args:
            request (HttpRequest): The given HttpRequest to execute.
            
        Returns:
            HttpResponse: The response of the HttpRequest.
            
        """
		
        response = requests.request(HttpMethodEnum.to_string(request.http_method), 
		                           request.query_url, 
								   headers=request.headers,
		                           params=request.query_parameters, 
                                   data=request.parameters, 
								   files=request.files,
								   auth=(request.username, request.password))
								   
        return self.convert_response(response, True)
    
    def convert_response(self, response, binary):
        """Converts the Response object of the HttpClient into an
        HttpResponse object.
       
        Args:
            response (dynamic): The original response object.
            
        Returns:
            HttpResponse: The converted HttpResponse object.
            
        """
        if binary == True:
            return HttpResponse(response.status_code, response.headers, response.content)
        else:
            return HttpResponse(response.status_code, response.headers, response.text)